"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[67341],{65660:(e,t,r)=>{r.r(t),r.d(t,{icon:()=>i});r(65043);var n=r(77724),l=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var n in r)({}).hasOwnProperty.call(r,n)&&(e[n]=r[n])}return e},a.apply(null,arguments)}var i=function(e){var t=e.title,r=e.titleId,i=function(e,t){if(null==e)return{};var r,n,l=function(e,t){if(null==e)return{};var r={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)>=0)continue;r[n]=e[n]}return r}(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(n=0;n<a.length;n++)r=a[n],t.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(l[r]=e[r])}return l}(e,l);return(0,n.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":r},i),t?(0,n.Y)("title",{id:r},t):null,(0,n.Y)("path",{d:"m11.276 29 .594 2H0l7.621-14.29.811 2.73L3.333 29h7.943ZM28.92 31l-4.987-16.598A16 16 0 0 0 8.688 3l1.8 6H8.4L6 1h2.607a18 18 0 0 1 17.241 12.828L31 31h-2.08Z"}),(0,n.Y)("path",{d:"M12.037 14.02 16.492 29h6.827l-2.333-7.849a10 10 0 0 0-8.949-7.13ZM9.35 12h2.05a12 12 0 0 1 11.503 8.581L26 31H15L9.35 12Z",className:"euiIcon__fillSecondary"}))}}}]);
//# sourceMappingURL=icon.app_code.29e18db1.chunk.js.map