"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import feast.serving.ServingService_pb2
import feast.types.EntityKey_pb2
import feast.types.Value_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ConnectorFeature(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REFERENCE_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    @property
    def reference(self) -> feast.serving.ServingService_pb2.FeatureReferenceV2: ...
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def value(self) -> feast.types.Value_pb2.Value: ...
    def __init__(
        self,
        *,
        reference: feast.serving.ServingService_pb2.FeatureReferenceV2 | None = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        value: feast.types.Value_pb2.Value | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["reference", b"reference", "timestamp", b"timestamp", "value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["reference", b"reference", "timestamp", b"timestamp", "value", b"value"]) -> None: ...

global___ConnectorFeature = ConnectorFeature

class ConnectorFeatureList(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURELIST_FIELD_NUMBER: builtins.int
    @property
    def featureList(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ConnectorFeature]: ...
    def __init__(
        self,
        *,
        featureList: collections.abc.Iterable[global___ConnectorFeature] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureList", b"featureList"]) -> None: ...

global___ConnectorFeatureList = ConnectorFeatureList

class OnlineReadRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENTITYKEYS_FIELD_NUMBER: builtins.int
    VIEW_FIELD_NUMBER: builtins.int
    FEATURES_FIELD_NUMBER: builtins.int
    @property
    def entityKeys(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.types.EntityKey_pb2.EntityKey]: ...
    view: builtins.str
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        entityKeys: collections.abc.Iterable[feast.types.EntityKey_pb2.EntityKey] | None = ...,
        view: builtins.str = ...,
        features: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["entityKeys", b"entityKeys", "features", b"features", "view", b"view"]) -> None: ...

global___OnlineReadRequest = OnlineReadRequest

class OnlineReadResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULTS_FIELD_NUMBER: builtins.int
    @property
    def results(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ConnectorFeatureList]: ...
    def __init__(
        self,
        *,
        results: collections.abc.Iterable[global___ConnectorFeatureList] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["results", b"results"]) -> None: ...

global___OnlineReadResponse = OnlineReadResponse
