"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[52634],{32433:(e,l,n)=>{n.r(l),n.d(l,{icon:()=>i});n(65043);var t=n(77724),r=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},a.apply(null,arguments)}var i=function(e){var l=e.title,n=e.titleId,i=function(e,l){if(null==e)return{};var n,t,r=function(e,l){if(null==e)return{};var n={};for(var t in e)if({}.hasOwnProperty.call(e,t)){if(l.indexOf(t)>=0)continue;n[t]=e[t]}return n}(e,l);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(t=0;t<a.length;t++)n=a[t],l.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(r[n]=e[n])}return r}(e,r);return(0,t.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":n},i),l?(0,t.Y)("title",{id:n},l):null,(0,t.Y)("path",{d:"M21 2.82 16 .038 11 2.82v2.289l5-2.782 5 2.782v-2.29Z",className:"euiIcon__fillSecondary"}),(0,t.Y)("path",{d:"M21 7.282 16 4.5l-5 2.782V9.57l5-2.781 5 2.781V7.282Z",className:"euiIcon__fillSecondary"}),(0,t.Y)("path",{d:"M7 5.045 2 7.827v15.577l14 7.788 14-7.788V7.827l-5-2.782v2.289l3 1.669v13.225l-12 6.676-12-6.676V9.003l3-1.669V5.045Z"}),(0,t.Y)("path",{fillRule:"evenodd",d:"M22 12.5 16 9l-6 3.5v7l6 3.5 6-3.5v-7Zm-9.974 1.205L16 11.387l3.974 2.318v4.59L16 20.613l-3.974-2.318v-4.59Z",className:"euiIcon__fillSecondary",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.app_fleet.e1e24753.chunk.js.map