"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[18002],{53601:(e,n,t)=>{t.r(n),t.d(n,{icon:()=>a});t(65043);var l=t(77724),r=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var n=1;n<arguments.length;n++){var t=arguments[n];for(var l in t)({}).hasOwnProperty.call(t,l)&&(e[l]=t[l])}return e},i.apply(null,arguments)}var a=function(e){var n=e.title,t=e.titleId,a=function(e,n){if(null==e)return{};var t,l,r=function(e,n){if(null==e)return{};var t={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(n.indexOf(l)>=0)continue;t[l]=e[l]}return t}(e,n);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(l=0;l<i.length;l++)t=i[l],n.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(r[t]=e[t])}return r}(e,r);return(0,l.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,fill:"none",viewBox:"0 0 16 16","aria-labelledby":t},a),n?(0,l.Y)("title",{id:t},n):null,(0,l.Y)("path",{fillRule:"evenodd",d:"M8 9.169 2 5.25v6.17l6 3.428 6-3.428V5.25L8 9.17ZM15 4v8l-7 4-7-4V4l7-4 7 4Zm-1.428.336L8 1.152 2.428 4.336 8 7.974l5.572-3.638Z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.analyzeEvent.3653139e.chunk.js.map