"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[83561],{44984:(a,e,t)=>{t.r(e),t.d(e,{icon:()=>i});t(65043);var r=t(77724),n=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var t=arguments[e];for(var r in t)({}).hasOwnProperty.call(t,r)&&(a[r]=t[r])}return a},l.apply(null,arguments)}var i=function(a){var e=a.title,t=a.titleId,i=function(a,e){if(null==a)return{};var t,r,n=function(a,e){if(null==a)return{};var t={};for(var r in a)if({}.hasOwnProperty.call(a,r)){if(e.indexOf(r)>=0)continue;t[r]=a[r]}return t}(a,e);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(a);for(r=0;r<l.length;r++)t=l[r],e.indexOf(t)>=0||{}.propertyIsEnumerable.call(a,t)&&(n[t]=a[t])}return n}(a,n);return(0,r.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":t},i),e?(0,r.Y)("title",{id:t},e):null,(0,r.Y)("path",{d:"M16 3v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v1Zm-1 0V2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v1h14Zm0 1H1v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4ZM4.5 6a.5.5 0 0 1 0 1H2.496a.5.5 0 1 1 0-1H4.5Zm9 0a.5.5 0 1 1 0 1h-6a.5.5 0 0 1 0-1h6Zm-9 3a.5.5 0 0 1 0 1H2.496a.5.5 0 1 1 0-1H4.5Zm9 0a.5.5 0 1 1 0 1h-6a.5.5 0 0 1 0-1h6Zm-9 3a.5.5 0 1 1 0 1H2.496a.5.5 0 1 1 0-1H4.5Zm9 0a.5.5 0 1 1 0 1h-6a.5.5 0 1 1 0-1h6Z"}))}}}]);
//# sourceMappingURL=icon.vis_table.4e86cae3.chunk.js.map