"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[46603],{13762:(e,t,r)=>{r.r(t),r.d(t,{icon:()=>l});r(65043);var n=r(77724),v=["title","titleId"];function h(){return h=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var n in r)({}).hasOwnProperty.call(r,n)&&(e[n]=r[n])}return e},h.apply(null,arguments)}var l=function(e){var t=e.title,r=e.titleId,l=function(e,t){if(null==e)return{};var r,n,v=function(e,t){if(null==e)return{};var r={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)>=0)continue;r[n]=e[n]}return r}(e,t);if(Object.getOwnPropertySymbols){var h=Object.getOwnPropertySymbols(e);for(n=0;n<h.length;n++)r=h[n],t.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(v[r]=e[r])}return v}(e,v);return(0,n.Y)("svg",h({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":r},l),t?(0,n.Y)("title",{id:r},t):null,(0,n.Y)("path",{d:"M8 2H7v3h1V4h3V3H8V2Zm-5 8H2v3h1v-1h3v-1H3v-1Zm4 0h1v1h3v1H8v1H7v-3Zm6 0h-1v3h1v-1h1v-1h-1v-1ZM5 6h1v1h3v1H6v1H5V6Zm6 0h-1v3h1V8h3V7h-3V6Zm1-4h1v1h1v1h-1v1h-1V2ZM6 3H2v1h4V3ZM2 7h2v1H2V7Z"}))}}}]);
//# sourceMappingURL=icon.pipeNoBreaks.656b5b65.chunk.js.map