"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[94154],{34753:(e,l,n)=>{n.r(l),n.d(l,{icon:()=>i});n(65043);var t=n(77724),r=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},a.apply(null,arguments)}var i=function(e){var l=e.title,n=e.titleId,i=function(e,l){if(null==e)return{};var n,t,r=function(e,l){if(null==e)return{};var n={};for(var t in e)if({}.hasOwnProperty.call(e,t)){if(l.indexOf(t)>=0)continue;n[t]=e[t]}return n}(e,l);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(t=0;t<a.length;t++)n=a[t],l.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(r[n]=e[n])}return r}(e,r);return(0,t.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,fill:"none",viewBox:"0 0 16 16","aria-labelledby":n},i),l?(0,t.Y)("title",{id:n},l):null,(0,t.Y)("path",{fillRule:"evenodd",d:"M2.232 14.043a.75.75 0 0 1-.025-1.06l4.591-4.81a.25.25 0 0 0 0-.346l-4.59-4.81a.75.75 0 0 1 1.085-1.035l4.59 4.81a1.75 1.75 0 0 1 0 2.416l-4.59 4.81a.75.75 0 0 1-1.06.024Z",clipRule:"evenodd"}),(0,t.Y)("path",{fillRule:"evenodd",d:"M8.232 14.043a.75.75 0 0 1-.025-1.06l4.591-4.81a.25.25 0 0 0 0-.346l-4.59-4.81a.75.75 0 0 1 1.085-1.035l4.59 4.81a1.75 1.75 0 0 1 0 2.416l-4.59 4.81a.75.75 0 0 1-1.06.024Z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.doubleArrowRight.28e17ef7.chunk.js.map