"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[76586],{88609:(e,t,n)=>{n.r(t),n.d(t,{icon:()=>i});n(65043);var l=n(77724),r=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var l in n)({}).hasOwnProperty.call(n,l)&&(e[l]=n[l])}return e},a.apply(null,arguments)}var i=function(e){var t=e.title,n=e.titleId,i=function(e,t){if(null==e)return{};var n,l,r=function(e,t){if(null==e)return{};var n={};for(var l in e)if({}.hasOwnProperty.call(e,l)){if(t.indexOf(l)>=0)continue;n[l]=e[l]}return n}(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(l=0;l<a.length;l++)n=a[l],t.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(r[n]=e[n])}return r}(e,r);return(0,l.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,fill:"none",viewBox:"0 0 16 16","aria-labelledby":n},i),t?(0,l.Y)("title",{id:n},t):null,(0,l.Y)("path",{fillRule:"evenodd",d:"M12.809 8.807a4.5 4.5 0 0 1-5.572-5.75A5 5 0 0 0 3.04 9.31a.5.5 0 0 1-.966.259 6 6 0 0 1 5.672-7.553 4.5 4.5 0 1 1 6.117 6.313 6.002 6.002 0 0 1-7.803 5.409.5.5 0 1 1 .303-.954 5.001 5.001 0 0 0 6.445-3.979ZM11.859 5H13.5a.5.5 0 0 0 0-1H12V1.5a.5.5 0 0 0-1 0v3a.5.5 0 0 0 .5.5h.36Z",clipRule:"evenodd"}),(0,l.Y)("path",{d:"m3.757 15.848-.776-2.897a1.5 1.5 0 0 1 1.06-1.838l2.898-.776a.5.5 0 1 1 .259.966l-2.898.776a.5.5 0 0 0-.353.613l.776 2.898a.5.5 0 0 1-.966.258Z"}))}}}]);
//# sourceMappingURL=icon.timeRefresh.b0eb9a09.chunk.js.map