"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[63233],{84632:(e,t,l)=>{l.r(t),l.d(t,{icon:()=>a});l(65043);var r=l(77724),n=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var l=arguments[t];for(var r in l)({}).hasOwnProperty.call(l,r)&&(e[r]=l[r])}return e},i.apply(null,arguments)}var a=function(e){var t=e.title,l=e.titleId,a=function(e,t){if(null==e)return{};var l,r,n=function(e,t){if(null==e)return{};var l={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;l[r]=e[r]}return l}(e,t);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(r=0;r<i.length;r++)l=i[r],t.indexOf(l)>=0||{}.propertyIsEnumerable.call(e,l)&&(n[l]=e[l])}return n}(e,n);return(0,r.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":l},a),t?(0,r.Y)("title",{id:l},t):null,(0,r.Y)("path",{d:"M9.75 12 16 32h10l-3.4-10.88A13 13 0 0 0 10.19 12h-.44Z",className:"euiIcon__fillNegative"}),(0,r.Y)("path",{fill:"#22A7F3",d:"M25.725 11.93A17 17 0 0 0 9.5 0H6l3.75 12h.44a13 13 0 0 1 12.41 9.12L26 32h6l-6.275-20.07Z"}),(0,r.Y)("path",{fill:"#0377CA",d:"M7.91 16.175 0 32h12.855z"}))}}}]);
//# sourceMappingURL=icon.logo_code.c9430d63.chunk.js.map