"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[96348],{46867:(e,l,t)=>{t.r(l),t.d(l,{icon:()=>i});t(65043);var r=t(77724),n=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var t=arguments[l];for(var r in t)({}).hasOwnProperty.call(t,r)&&(e[r]=t[r])}return e},a.apply(null,arguments)}var i=function(e){var l=e.title,t=e.titleId,i=function(e,l){if(null==e)return{};var t,r,n=function(e,l){if(null==e)return{};var t={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(l.indexOf(r)>=0)continue;t[r]=e[r]}return t}(e,l);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(r=0;r<a.length;r++)t=a[r],l.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(n[t]=e[t])}return n}(e,n);return(0,r.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":t},i),l?(0,r.Y)("title",{id:t},l):null,(0,r.Y)("g",{fillRule:"evenodd"},(0,r.Y)("path",{d:"M9.82 5.116a.5.5 0 0 0-.704.704l.064.064L11.719 8l-2.54 2.116a.5.5 0 0 0-.114.63l.05.074a.5.5 0 0 0 .63.115l.075-.05 3-2.5a.5.5 0 0 0 .071-.697l-.07-.072-3-2.5ZM6.18 5.116a.5.5 0 0 1 .704.704l-.064.064L4.281 8l2.54 2.116a.5.5 0 0 1 .114.63l-.05.074a.5.5 0 0 1-.63.115l-.075-.05-3-2.5a.5.5 0 0 1-.071-.697l.07-.072 3-2.5Z"})))}}}]);
//# sourceMappingURL=icon.tokenRange.d4e867a2.chunk.js.map