from feast import Feature

{{ name }} = Feature(
    name="{{ name }}",
    dtype={{ dtype }},
    {% if description %}description="{{ description }}",{% endif %}
    {% if tags %}labels={{ tags }},{% endif %}
) 