from feast import FeatureService

{{ name }} = FeatureService(
    name="{{ name }}",
    features=[{{ features }}],
    {% if tags %}tags={{ tags }},{% endif %}
    {% if description %}description="{{ description }}",{% endif %}
    {% if logging_config %}logging_config={{ logging_config }},{% endif %}
) 