DROP TABLE IF EXISTS REGISTRY_PATH."DATA_SOURCES";

DROP TABLE IF EXISTS REGISTRY_PATH."ENTITIES";

DROP TABLE IF EXISTS REGISTRY_PATH."FEAST_METADATA";

DROP TABLE IF EXISTS REGISTRY_PATH."FEATURE_SERVICES";

DROP TABLE IF EXISTS REGISTRY_PATH."FEATURE_VIEWS";

DROP TABLE IF EXISTS REGISTRY_PATH."MANAGED_INFRA";

DROP TABLE IF EXISTS REGISTRY_PATH."ON_DEMAND_FEATURE_VIEWS";

DROP TABLE IF EXISTS REGISTRY_PATH."SAVED_DATASETS";

DROP TABLE IF EXISTS REGISTRY_PATH."STREAM_FEATURE_VIEWS";

DROP TABLE IF EXISTS REGISTRY_PATH."VALIDATION_REFERENCES"

DROP TABLE IF EXISTS REGISTRY_PATH."PERMISSIONS"
