"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[7397],{4780:(e,t,l)=>{l.r(t),l.d(t,{icon:()=>a});l(65043);var n=l(77724),r=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var l=arguments[t];for(var n in l)({}).hasOwnProperty.call(l,n)&&(e[n]=l[n])}return e},i.apply(null,arguments)}var a=function(e){var t=e.title,l=e.titleId,a=function(e,t){if(null==e)return{};var l,n,r=function(e,t){if(null==e)return{};var l={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)>=0)continue;l[n]=e[n]}return l}(e,t);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(n=0;n<i.length;n++)l=i[n],t.indexOf(l)>=0||{}.propertyIsEnumerable.call(e,l)&&(r[l]=e[l])}return r}(e,r);return(0,n.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":l},a),t?(0,n.Y)("title",{id:l},t):null,(0,n.Y)("g",{fill:"none",fillRule:"evenodd"},(0,n.Y)("path",{fill:"#00BFB3",d:"M0 22c0 5.522 4.478 10 10 10V12C4.478 12 0 16.478 0 22"}),(0,n.Y)("path",{d:"M10 12v10h10c0-5.522-4.478-10-10-10",className:"euiIcon__fillNegative"}),(0,n.Y)("path",{fill:"#F04E98",d:"M10 0v9c7.168 0 13 5.832 13 13h9C32 9.85 22.15 0 10 0"})))}}}]);
//# sourceMappingURL=icon.logo_business_analytics.f4c36905.chunk.js.map