"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[4898],{18401:(e,l,n)=>{n.r(l),n.d(l,{icon:()=>a});n(65043);var t=n(77724),r=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var n=arguments[l];for(var t in n)({}).hasOwnProperty.call(n,t)&&(e[t]=n[t])}return e},i.apply(null,arguments)}var a=function(e){var l=e.title,n=e.titleId,a=function(e,l){if(null==e)return{};var n,t,r=function(e,l){if(null==e)return{};var n={};for(var t in e)if({}.hasOwnProperty.call(e,t)){if(l.indexOf(t)>=0)continue;n[t]=e[t]}return n}(e,l);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(t=0;t<i.length;t++)n=i[t],l.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(r[n]=e[n])}return r}(e,r);return(0,t.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,fill:"none",viewBox:"0 0 16 16","aria-labelledby":n},a),l?(0,t.Y)("title",{id:n},l):null,(0,t.Y)("path",{fillRule:"evenodd",d:"M14 11.42V4.58L8 1.152 2 4.58v6.84l6 3.428 6-3.428zM8 16l7-4V4L8 0 1 4v8l7 4zM8 4.607l3 1.714V9.68l-3 1.714-3-1.714V6.32l3-1.714zm4 1.134v4.518l-4 2.286-4-2.286V5.741l4-2.286 4 2.286z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.kubernetesNode.98e86d53.chunk.js.map