"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[20765],{8804:(e,r,t)=>{t.r(r),t.d(r,{icon:()=>i});t(65043);var n=t(77724),a=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var r=1;r<arguments.length;r++){var t=arguments[r];for(var n in t)({}).hasOwnProperty.call(t,n)&&(e[n]=t[n])}return e},l.apply(null,arguments)}var i=function(e){var r=e.title,t=e.titleId,i=function(e,r){if(null==e)return{};var t,n,a=function(e,r){if(null==e)return{};var t={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(r.indexOf(n)>=0)continue;t[n]=e[n]}return t}(e,r);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(n=0;n<l.length;n++)t=l[n],r.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(a[t]=e[t])}return a}(e,a);return(0,n.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":t},i),r?(0,n.Y)("title",{id:t},r):null,(0,n.Y)("g",{fillRule:"evenodd",transform:"translate(0 2)"},(0,n.Y)("path",{fillRule:"nonzero",d:"M2 6a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1H2Zm13 2v3a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V8a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1Zm1-3V2a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v3c0 .601.271 1.133.69 1.5C.271 6.867 0 7.399 0 8v3a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V8c0-.601-.271-1.133-.689-1.5.418-.367.689-.899.689-1.5Z"}),(0,n.Y)("circle",{cx:4.5,cy:9.5,r:1.5}),(0,n.Y)("circle",{cx:4.5,cy:3.5,r:1.5}),(0,n.Y)("path",{d:"M12 8h1v3h-1zM10 8h1v3h-1zM12 2h1v3h-1zM10 2h1v3h-1z"})))}}}]);
//# sourceMappingURL=icon.storage.b8dd9df4.chunk.js.map