"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[35175],{24558:(e,t,n)=>{n.r(t),n.d(t,{icon:()=>i});n(65043);var r=n(77724),l=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)({}).hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},a.apply(null,arguments)}var i=function(e){var t=e.title,n=e.titleId,i=function(e,t){if(null==e)return{};var n,r,l=function(e,t){if(null==e)return{};var n={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(r=0;r<a.length;r++)n=a[r],t.indexOf(n)>=0||{}.propertyIsEnumerable.call(e,n)&&(l[n]=e[n])}return l}(e,l);return(0,r.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":n},i),t?(0,r.Y)("title",{id:n},t):null,(0,r.Y)("path",{d:"M4 4h6v2H4zM22 4h6v2h-6zM4 22h6v2H4z",className:"euiIcon__fillSecondary"}),(0,r.Y)("path",{d:"M0 14h14V0H0v14zM2 2h10v10H2V2zm16-2v14h14V0H18zm12 12H20V2h10v10zM0 32h14V18H0v14zm2-12h10v10H2V20zm16 12h14V18H18v14zm2-12h10v10H20V20z"}),(0,r.Y)("path",{d:"M22 22h6v2h-6z",className:"euiIcon__fillSecondary"}))}}}]);
//# sourceMappingURL=icon.app_spaces.8b34d39a.chunk.js.map