"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[51065],{99128:(e,l,t)=>{t.r(l),t.d(l,{icon:()=>a});t(65043);var n=t(77724),r=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var t=arguments[l];for(var n in t)({}).hasOwnProperty.call(t,n)&&(e[n]=t[n])}return e},i.apply(null,arguments)}var a=function(e){var l=e.title,t=e.titleId,a=function(e,l){if(null==e)return{};var t,n,r=function(e,l){if(null==e)return{};var t={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(l.indexOf(n)>=0)continue;t[n]=e[n]}return t}(e,l);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(n=0;n<i.length;n++)t=i[n],l.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(r[t]=e[t])}return r}(e,r);return(0,n.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":t},a),l?(0,n.Y)("title",{id:t},l):null,(0,n.Y)("g",{fill:"none",fillRule:"evenodd"},(0,n.Y)("path",{fill:"#F04E98",d:"M2 32h28V20l-6.465-6.465a5 5 0 0 0-7.07 0L2 28v4Z"}),(0,n.Y)("path",{d:"m16.465 13.535-3.536 3.536a9.965 9.965 0 0 0 7.07 2.93 9.965 9.965 0 0 0 7.072-2.93l-3.536-3.536a5 5 0 0 0-7.07 0",className:"euiIcon__fillNegative"}),(0,n.Y)("path",{fill:"#FEC514",d:"M14.343 11.414A7.951 7.951 0 0 1 20 9.071c2.137 0 4.146.832 5.657 2.343l3.207 3.207A9.955 9.955 0 0 0 30 10.001c0-5.524-4.477-10-10-10-5.522 0-10 4.476-10 10 0 1.667.414 3.237 1.137 4.62l3.206-3.207Z"})))}}}]);
//# sourceMappingURL=icon.logo_metrics.f5fc19fc.chunk.js.map