"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[35861],{89764:(l,e,t)=>{t.r(e),t.d(e,{icon:()=>a});t(65043);var r=t(77724),n=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(l){for(var e=1;e<arguments.length;e++){var t=arguments[e];for(var r in t)({}).hasOwnProperty.call(t,r)&&(l[r]=t[r])}return l},i.apply(null,arguments)}var a=function(l){var e=l.title,t=l.titleId,a=function(l,e){if(null==l)return{};var t,r,n=function(l,e){if(null==l)return{};var t={};for(var r in l)if({}.hasOwnProperty.call(l,r)){if(e.indexOf(r)>=0)continue;t[r]=l[r]}return t}(l,e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(l);for(r=0;r<i.length;r++)t=i[r],e.indexOf(t)>=0||{}.propertyIsEnumerable.call(l,t)&&(n[t]=l[t])}return n}(l,n);return(0,r.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":t},a),e?(0,r.Y)("title",{id:t},e):null,(0,r.Y)("path",{fillRule:"evenodd",d:"M7.808 10.197H6.796L5.859 13H4.485l.937-2.803H3.966l.219-1.25h1.647l.608-1.805H4.991l.226-1.251h1.64l.95-2.844h1.368l-.95 2.844h1.018l.95-2.844h1.374l-.95 2.844h1.51l-.218 1.25h-1.702l-.608 1.805h1.497l-.219 1.251H9.182L8.252 13H6.878l.93-2.803zm-.602-1.25h1.012l.615-1.805H7.814l-.608 1.804z"}))}}}]);
//# sourceMappingURL=icon.number.702e9f09.chunk.js.map