"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[66084],{96923:(e,l,t)=>{t.r(l),t.d(l,{icon:()=>i});t(65043);var r=t(77724),n=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var l=1;l<arguments.length;l++){var t=arguments[l];for(var r in t)({}).hasOwnProperty.call(t,r)&&(e[r]=t[r])}return e},a.apply(null,arguments)}var i=function(e){var l=e.title,t=e.titleId,i=function(e,l){if(null==e)return{};var t,r,n=function(e,l){if(null==e)return{};var t={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(l.indexOf(r)>=0)continue;t[r]=e[r]}return t}(e,l);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(r=0;r<a.length;r++)t=a[r],l.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(n[t]=e[t])}return n}(e,n);return(0,r.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":t},i),l?(0,r.Y)("title",{id:t},l):null,(0,r.Y)("path",{d:"M4.854 5.146a.5.5 0 1 0-.708.708L6.793 8.5l-2.647 2.646a.5.5 0 0 0 .708.708L7.5 9.207l2.646 2.647a.5.5 0 0 0 .708-.708L8.207 8.5l2.647-2.646a.5.5 0 0 0-.708-.708L7.5 7.793 4.854 5.146Z"}),(0,r.Y)("path",{fillRule:"evenodd",d:"M5 1a1 1 0 0 0-.707.293l-4 4A1 1 0 0 0 0 6v5a1 1 0 0 0 .293.707l4 4A1 1 0 0 0 5 16h5a1 1 0 0 0 .707-.293l4-4A1 1 0 0 0 15 11V6a1 1 0 0 0-.293-.707l-4-4A1 1 0 0 0 10 1H5Zm5 1H5L1 6v5l4 4h5l4-4V6l-4-4Z"}))}}}]);
//# sourceMappingURL=icon.error.0a5d8dd9.chunk.js.map