"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class PushRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class FeaturesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    FEATURES_FIELD_NUMBER: builtins.int
    STREAM_FEATURE_VIEW_FIELD_NUMBER: builtins.int
    ALLOW_REGISTRY_CACHE_FIELD_NUMBER: builtins.int
    TO_FIELD_NUMBER: builtins.int
    @property
    def features(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    stream_feature_view: builtins.str
    allow_registry_cache: builtins.bool
    to: builtins.str
    def __init__(
        self,
        *,
        features: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        stream_feature_view: builtins.str = ...,
        allow_registry_cache: builtins.bool = ...,
        to: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_registry_cache", b"allow_registry_cache", "features", b"features", "stream_feature_view", b"stream_feature_view", "to", b"to"]) -> None: ...

global___PushRequest = PushRequest

class PushResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STATUS_FIELD_NUMBER: builtins.int
    status: builtins.bool
    def __init__(
        self,
        *,
        status: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["status", b"status"]) -> None: ...

global___PushResponse = PushResponse

class WriteToOnlineStoreRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class FeaturesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    FEATURES_FIELD_NUMBER: builtins.int
    FEATURE_VIEW_NAME_FIELD_NUMBER: builtins.int
    ALLOW_REGISTRY_CACHE_FIELD_NUMBER: builtins.int
    @property
    def features(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    feature_view_name: builtins.str
    allow_registry_cache: builtins.bool
    def __init__(
        self,
        *,
        features: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        feature_view_name: builtins.str = ...,
        allow_registry_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_registry_cache", b"allow_registry_cache", "feature_view_name", b"feature_view_name", "features", b"features"]) -> None: ...

global___WriteToOnlineStoreRequest = WriteToOnlineStoreRequest

class WriteToOnlineStoreResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STATUS_FIELD_NUMBER: builtins.int
    status: builtins.bool
    def __init__(
        self,
        *,
        status: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["status", b"status"]) -> None: ...

global___WriteToOnlineStoreResponse = WriteToOnlineStoreResponse
