"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[27433],{16784:(e,t,r)=>{r.r(t),r.d(t,{icon:()=>i});r(65043);var n=r(77724),l=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var n in r)({}).hasOwnProperty.call(r,n)&&(e[n]=r[n])}return e},a.apply(null,arguments)}var i=function(e){var t=e.title,r=e.titleId,i=function(e,t){if(null==e)return{};var r,n,l=function(e,t){if(null==e)return{};var r={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)>=0)continue;r[n]=e[n]}return r}(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(n=0;n<a.length;n++)r=a[n],t.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(l[r]=e[r])}return l}(e,l);return(0,n.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":r},i),t?(0,n.Y)("title",{id:r},t):null,(0,n.Y)("path",{d:"M25 2h-5V0h-2v2h-3V0h-2v2h-3V0H8v2H3v26h22V2Zm-2 24H5V4h3v2h2V4h3v2h2V4h3v2h2V4h3v22Z"}),(0,n.Y)("path",{d:"M27 7v23H8v2h21V7z"}),(0,n.Y)("path",{d:"M8 12h12v2H8zM8 17h12v2H8z",className:"euiIcon__fillSecondary"}))}}}]);
//# sourceMappingURL=icon.app_notebook.04b40a3c.chunk.js.map