"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[94856],{42871:(e,t,r)=>{r.r(t),r.d(t,{icon:()=>i});r(65043);var a=r(77724),n=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var a in r)({}).hasOwnProperty.call(r,a)&&(e[a]=r[a])}return e},l.apply(null,arguments)}var i=function(e){var t=e.title,r=e.titleId,i=function(e,t){if(null==e)return{};var r,a,n=function(e,t){if(null==e)return{};var r={};for(var a in e)if({}.hasOwnProperty.call(e,a)){if(t.indexOf(a)>=0)continue;r[a]=e[a]}return r}(e,t);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(a=0;a<l.length;a++)r=l[a],t.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}(e,n);return(0,a.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":r},i),t?(0,a.Y)("title",{id:r},t):null,(0,a.Y)("path",{d:"M.5 1a.5.5 0 0 0 0 1h9a.5.5 0 0 0 0-1h-9ZM16 11.5a4.5 4.5 0 1 1-9 0 4.5 4.5 0 0 1 9 0Zm-4-3a.5.5 0 0 0-1 0V11H8.5a.5.5 0 0 0 0 1H11v2.5a.5.5 0 0 0 1 0V12h2.5a.5.5 0 0 0 0-1H12V8.5ZM2 4.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5ZM4.5 7a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1Z"}))}}}]);
//# sourceMappingURL=icon.filterInclude.98818d7c.chunk.js.map