"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[61569],{11280:(e,t,l)=>{l.r(t),l.d(t,{icon:()=>i});l(65043);var r=l(77724),a=["title","titleId"];function n(){return n=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var l=arguments[t];for(var r in l)({}).hasOwnProperty.call(l,r)&&(e[r]=l[r])}return e},n.apply(null,arguments)}var i=function(e){var t=e.title,l=e.titleId,i=function(e,t){if(null==e)return{};var l,r,a=function(e,t){if(null==e)return{};var l={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;l[r]=e[r]}return l}(e,t);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);for(r=0;r<n.length;r++)l=n[r],t.indexOf(l)>=0||{}.propertyIsEnumerable.call(e,l)&&(a[l]=e[l])}return a}(e,a);return(0,r.Y)("svg",n({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":l},i),t?(0,r.Y)("title",{id:l},t):null,(0,r.Y)("path",{d:"M10.146 5.854a.5.5 0 0 1 .708-.708l2 2a.5.5 0 0 1 0 .708l-2 2a.5.5 0 0 1-.708-.708L11.293 8H7.5a.5.5 0 0 1 0-1h3.793l-1.147-1.146Z"}),(0,r.Y)("path",{fillRule:"evenodd",d:"M2 0h11a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2Zm2.707 1H5v1.293l-4 4V4.707L4.707 1ZM6 14V1h7a1 1 0 0 1 1 1v11a1 1 0 0 1-1 1H6Zm-1-1.293V14H3.707L5 12.707Zm0-3v1.586L2.293 14H2a1 1 0 0 1-.838-.454L5 9.707Zm0-3v1.586l-4 4v-1.586l4-4Zm0-3v1.586l-4 4V7.707l4-4ZM3.293 1H2a1 1 0 0 0-1 1v1.293L3.293 1Z"}))}}}]);
//# sourceMappingURL=icon.transitionLeftIn.9fbd3c22.chunk.js.map