"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[66036],{82155:(a,e,t)=>{t.r(e),t.d(e,{icon:()=>i});t(65043);var r=t(77724),n=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var t=arguments[e];for(var r in t)({}).hasOwnProperty.call(t,r)&&(a[r]=t[r])}return a},l.apply(null,arguments)}var i=function(a){var e=a.title,t=a.titleId,i=function(a,e){if(null==a)return{};var t,r,n=function(a,e){if(null==a)return{};var t={};for(var r in a)if({}.hasOwnProperty.call(a,r)){if(e.indexOf(r)>=0)continue;t[r]=a[r]}return t}(a,e);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(a);for(r=0;r<l.length;r++)t=l[r],e.indexOf(t)>=0||{}.propertyIsEnumerable.call(a,t)&&(n[t]=a[t])}return n}(a,n);return(0,r.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":t},i),e?(0,r.Y)("title",{id:t},e):null,(0,r.Y)("path",{d:"M29 12a3 3 0 0 0-3 3h-4a3 3 0 0 0-3-3h-6a3 3 0 0 0-3 3H6a3 3 0 0 0-3-3H0v14h3a3 3 0 0 0 3-3h4a3 3 0 0 0 3 3h6a3 3 0 0 0 3-3h4a3 3 0 0 0 3 3h3V12h-3ZM3 24H2V14h1a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1Zm17-3v2a1 1 0 0 1-1 1h-6a1 1 0 0 1-1-1v-2H6v-4h6v-2a1 1 0 0 1 1-1h6a1 1 0 0 1 1 1v2h6v4h-6Zm10 3h-1a1 1 0 0 1-1-1v-8a1 1 0 0 1 1-1h1v10Z"}),(0,r.Y)("path",{d:"M22 6H10v2h5v2h2V8h5z",className:"euiIcon__fillSecondary"}))}}}]);
//# sourceMappingURL=icon.app_pipeline.b1502b09.chunk.js.map