"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[69270],{94693:(e,t,r)=>{r.r(t),r.d(t,{icon:()=>l});r(65043);var n=r(77724),v=["title","titleId"];function h(){return h=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var n in r)({}).hasOwnProperty.call(r,n)&&(e[n]=r[n])}return e},h.apply(null,arguments)}var l=function(e){var t=e.title,r=e.titleId,l=function(e,t){if(null==e)return{};var r,n,v=function(e,t){if(null==e)return{};var r={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)>=0)continue;r[n]=e[n]}return r}(e,t);if(Object.getOwnPropertySymbols){var h=Object.getOwnPropertySymbols(e);for(n=0;n<h.length;n++)r=h[n],t.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(v[r]=e[r])}return v}(e,v);return(0,n.Y)("svg",h({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":r},l),t?(0,n.Y)("title",{id:r},t):null,(0,n.Y)("path",{d:"M1 5V1h4v4H1Zm3-1V2H2v2h2Zm2 1V1h4v4H6Zm3-1V2H7v2h2Zm2 1V1h4v4h-4Zm1-1h2V2h-2v2ZM1 10V6h4v4H1Zm3-1V7H2v2h2Zm2 1V6h4v4H6Zm3-1V7H7v2h2Zm2 1V6h4v4h-4Zm3-1V7h-2v2h2ZM1 15v-4h4v4H1Zm1-1h2v-2H2v2Zm4 1v-4h4v4H6Zm1-1h2v-2H7v2Zm4 1v-4h4v4h-4Zm1-1h2v-2h-2v2Z"}))}}}]);
//# sourceMappingURL=icon.grid.982e8626.chunk.js.map