"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[40605],{49172:(e,n,r)=>{r.r(n),r.d(n,{icon:()=>h});r(65043);var t=r(77724),v=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(e){for(var n=1;n<arguments.length;n++){var r=arguments[n];for(var t in r)({}).hasOwnProperty.call(r,t)&&(e[t]=r[t])}return e},l.apply(null,arguments)}var h=function(e){var n=e.title,r=e.titleId,h=function(e,n){if(null==e)return{};var r,t,v=function(e,n){if(null==e)return{};var r={};for(var t in e)if({}.hasOwnProperty.call(e,t)){if(n.indexOf(t)>=0)continue;r[t]=e[t]}return r}(e,n);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(t=0;t<l.length;t++)r=l[t],n.indexOf(r)>=0||{}.propertyIsEnumerable.call(e,r)&&(v[r]=e[r])}return v}(e,v);return(0,t.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 16 16","aria-labelledby":r},h),n?(0,t.Y)("title",{id:r},n):null,(0,t.Y)("path",{fillRule:"evenodd",d:"M0 3h16v11H0V3Zm15 10V4H1v9h14ZM2 5h2v1H2V5Zm2 7v-1H2v1h2Zm10-1v1h-2v-1h2Zm-3 1v-1H5v1h6ZM5 5h1v1H5V5ZM3 7H2v1h1V7Zm3 0h1v1H6V7ZM3 9H2v1h1V9Zm3 0h1v1H6V9Zm2-4H7v1h1V5ZM4 7h1v1H4V7Zm5 0H8v1h1V7Zm1 0h1v1h-1V7ZM5 9H4v1h1V9Zm3 0h1v1H8V9Zm3 0h-1v1h1V9ZM9 5h1v1H9V5Zm3 0h-1v1h1V5Zm1 0h1v1h-1V5Zm1 2h-2v3h2V7Z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.keyboard.8a09b24d.chunk.js.map