"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[52164],{9067:(e,n,t)=>{t.r(n),t.d(n,{icon:()=>i});t(65043);var r=t(77724),l=["title","titleId"];function a(){return a=Object.assign?Object.assign.bind():function(e){for(var n=1;n<arguments.length;n++){var t=arguments[n];for(var r in t)({}).hasOwnProperty.call(t,r)&&(e[r]=t[r])}return e},a.apply(null,arguments)}var i=function(e){var n=e.title,t=e.titleId,i=function(e,n){if(null==e)return{};var t,r,l=function(e,n){if(null==e)return{};var t={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(n.indexOf(r)>=0)continue;t[r]=e[r]}return t}(e,n);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(r=0;r<a.length;r++)t=a[r],n.indexOf(t)>=0||{}.propertyIsEnumerable.call(e,t)&&(l[t]=e[t])}return l}(e,l);return(0,r.Y)("svg",a({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,fill:"none",viewBox:"0 0 16 16","aria-labelledby":t},i),n?(0,r.Y)("title",{id:t},n):null,(0,r.Y)("path",{fillRule:"evenodd",d:"M4.5 7a2.5 2.5 0 1 0 0-5 2.5 2.5 0 0 0 0 5zm0-6a3.5 3.5 0 0 1 3.465 3h3.12a1.5 1.5 0 1 1 0 1h-3.12a3.482 3.482 0 0 1-.662 1.596l2.1 2.1A3.5 3.5 0 1 1 8.036 12h-3.12a1.5 1.5 0 1 1 0-.999h3.12a3.482 3.482 0 0 1 .662-1.596l-2.1-2.1A3.5 3.5 0 1 1 4.5 1zM12 4.5a.5.5 0 1 0 1 0 .5.5 0 0 0-1 0zm-.5 9.5a2.5 2.5 0 1 1 0-5 2.5 2.5 0 0 1 0 5zM4 11.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0z",clipRule:"evenodd"}))}}}]);
//# sourceMappingURL=icon.cluster.8b49b7f9.chunk.js.map