# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/Policy.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x66\x65\x61st/core/Policy.proto\x12\nfeast.core\"\xc5\x02\n\x06Policy\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x38\n\x11role_based_policy\x18\x03 \x01(\x0b\x32\x1b.feast.core.RoleBasedPolicyH\x00\x12:\n\x12group_based_policy\x18\x04 \x01(\x0b\x32\x1c.feast.core.GroupBasedPolicyH\x00\x12\x42\n\x16namespace_based_policy\x18\x05 \x01(\x0b\x32 .feast.core.NamespaceBasedPolicyH\x00\x12S\n\x1f\x63ombined_group_namespace_policy\x18\x06 \x01(\x0b\x32(.feast.core.CombinedGroupNamespacePolicyH\x00\x42\r\n\x0bpolicy_type\" \n\x0fRoleBasedPolicy\x12\r\n\x05roles\x18\x01 \x03(\t\"\"\n\x10GroupBasedPolicy\x12\x0e\n\x06groups\x18\x01 \x03(\t\"*\n\x14NamespaceBasedPolicy\x12\x12\n\nnamespaces\x18\x01 \x03(\t\"B\n\x1c\x43ombinedGroupNamespacePolicy\x12\x0e\n\x06groups\x18\x01 \x03(\t\x12\x12\n\nnamespaces\x18\x02 \x03(\tBP\n\x10\x66\x65\x61st.proto.coreB\x0bPolicyProtoZ/github.com/feast-dev/feast/go/protos/feast/coreb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.Policy_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\020feast.proto.coreB\013PolicyProtoZ/github.com/feast-dev/feast/go/protos/feast/core'
  _globals['_POLICY']._serialized_start=40
  _globals['_POLICY']._serialized_end=365
  _globals['_ROLEBASEDPOLICY']._serialized_start=367
  _globals['_ROLEBASEDPOLICY']._serialized_end=399
  _globals['_GROUPBASEDPOLICY']._serialized_start=401
  _globals['_GROUPBASEDPOLICY']._serialized_end=435
  _globals['_NAMESPACEBASEDPOLICY']._serialized_start=437
  _globals['_NAMESPACEBASEDPOLICY']._serialized_end=479
  _globals['_COMBINEDGROUPNAMESPACEPOLICY']._serialized_start=481
  _globals['_COMBINEDGROUPNAMESPACEPOLICY']._serialized_end=547
# @@protoc_insertion_point(module_scope)
