"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[8135],{52534:(t,e,r)=>{r.r(e),r.d(e,{icon:()=>l});r(65043);var n=r(77724),v=["title","titleId"];function h(){return h=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var r=arguments[e];for(var n in r)({}).hasOwnProperty.call(r,n)&&(t[n]=r[n])}return t},h.apply(null,arguments)}var l=function(t){var e=t.title,r=t.titleId,l=function(t,e){if(null==t)return{};var r,n,v=function(t,e){if(null==t)return{};var r={};for(var n in t)if({}.hasOwnProperty.call(t,n)){if(e.indexOf(n)>=0)continue;r[n]=t[n]}return r}(t,e);if(Object.getOwnPropertySymbols){var h=Object.getOwnPropertySymbols(t);for(n=0;n<h.length;n++)r=h[n],e.indexOf(r)>=0||{}.propertyIsEnumerable.call(t,r)&&(v[r]=t[r])}return v}(t,v);return(0,n.Y)("svg",h({xmlns:"http://www.w3.org/2000/svg",width:16,height:16,viewBox:"0 0 16 16","aria-labelledby":r},l),e?(0,n.Y)("title",{id:r},e):null,(0,n.Y)("path",{d:"M3 13V2h11V1H2v13h2v-1H3Z"}),(0,n.Y)("path",{d:"M11.999 5H6V4h5.999v1ZM4 4v1h1V4H4Zm2 3v1h4V7H6ZM4 7v1h1V7H4Zm2 3v1h2v-1H6Zm-2 0v1h1v-1H4Zm10-7h-1v1h1V3Zm-1 2h1v1h-1V5Zm1 2h-1v1h1V7Zm-1 2h1v1h-1V9Zm1 2h-1v1h1v-1Zm-1 2h1v1h-1v-1Zm-7 0H5v1h1v-1Zm1 0h1v1H7v-1Zm3 0H9v1h1v-1Zm1 0h1v1h-1v-1Z"}))}}}]);
//# sourceMappingURL=icon.indexTemporary.23f46f85.chunk.js.map