"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[52499],{90914:(e,t,l)=>{l.r(t),l.d(t,{icon:()=>a});l(65043);var n=l(77724),r=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var l=arguments[t];for(var n in l)({}).hasOwnProperty.call(l,n)&&(e[n]=l[n])}return e},i.apply(null,arguments)}var a=function(e){var t=e.title,l=e.titleId,a=function(e,t){if(null==e)return{};var l,n,r=function(e,t){if(null==e)return{};var l={};for(var n in e)if({}.hasOwnProperty.call(e,n)){if(t.indexOf(n)>=0)continue;l[n]=e[n]}return l}(e,t);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(e);for(n=0;n<i.length;n++)l=i[n],t.indexOf(l)>=0||{}.propertyIsEnumerable.call(e,l)&&(r[l]=e[l])}return r}(e,r);return(0,n.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":l},a),t?(0,n.Y)("title",{id:l},t):null,(0,n.Y)("g",{fill:"none",fillRule:"evenodd"},(0,n.Y)("path",{d:"M2 16c0 1.384.194 2.72.524 4H22a4 4 0 0 0 0-8H2.524A15.984 15.984 0 0 0 2 16",className:"euiIcon__fillNegative"}),(0,n.Y)("path",{fill:"#FEC514",d:"M28.924 7.662A15.381 15.381 0 0 0 30.48 6C27.547 2.346 23.05 0 18 0 11.679 0 6.239 3.678 3.644 9H25.51a5.039 5.039 0 0 0 3.413-1.338"}),(0,n.Y)("path",{fill:"#00BFB3",d:"M25.51 23H3.645C6.24 28.323 11.679 32 18 32c5.05 0 9.547-2.346 12.48-6a15.381 15.381 0 0 0-1.556-1.662A5.034 5.034 0 0 0 25.51 23"})))}}}]);
//# sourceMappingURL=icon.logo_elasticsearch.3a82ba08.chunk.js.map