"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[34129],{83784:(a,e,t)=>{t.r(e),t.d(e,{icon:()=>i});t(65043);var r=t(77724),n=["title","titleId"];function l(){return l=Object.assign?Object.assign.bind():function(a){for(var e=1;e<arguments.length;e++){var t=arguments[e];for(var r in t)({}).hasOwnProperty.call(t,r)&&(a[r]=t[r])}return a},l.apply(null,arguments)}var i=function(a){var e=a.title,t=a.titleId,i=function(a,e){if(null==a)return{};var t,r,n=function(a,e){if(null==a)return{};var t={};for(var r in a)if({}.hasOwnProperty.call(a,r)){if(e.indexOf(r)>=0)continue;t[r]=a[r]}return t}(a,e);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(a);for(r=0;r<l.length;r++)t=l[r],e.indexOf(t)>=0||{}.propertyIsEnumerable.call(a,t)&&(n[t]=a[t])}return n}(a,n);return(0,r.Y)("svg",l({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":t},i),e?(0,r.Y)("title",{id:t},e):null,(0,r.Y)("path",{d:"M29 9H3a3 3 0 0 1-3-3V3a3 3 0 0 1 3-3h26a3 3 0 0 1 3 3v3a3 3 0 0 1-3 3ZM3 2a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h26a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H3Z"}),(0,r.Y)("path",{d:"M29 32H3a3 3 0 0 1-3-3V14a3 3 0 0 1 3-3h26a3 3 0 0 1 3 3v15a3 3 0 0 1-3 3ZM3 13a1 1 0 0 0-1 1v15a1 1 0 0 0 1 1h26a1 1 0 0 0 1-1V14a1 1 0 0 0-1-1H3Z",className:"euiIcon__fillSecondary"}),(0,r.Y)("path",{d:"m7.29 17.71 3.3 3.29-3.3 3.29 1.42 1.42 4.7-4.71-4.7-4.71zM15 24h9v2h-9z"}))}}}]);
//# sourceMappingURL=icon.app_console.1e5f8deb.chunk.js.map