"use strict";(self.webpackChunkexample_feast_ui=self.webpackChunkexample_feast_ui||[]).push([[60901],{8244:(l,e,t)=>{t.r(e),t.d(e,{icon:()=>a});t(65043);var r=t(77724),n=["title","titleId"];function i(){return i=Object.assign?Object.assign.bind():function(l){for(var e=1;e<arguments.length;e++){var t=arguments[e];for(var r in t)({}).hasOwnProperty.call(t,r)&&(l[r]=t[r])}return l},i.apply(null,arguments)}var a=function(l){var e=l.title,t=l.titleId,a=function(l,e){if(null==l)return{};var t,r,n=function(l,e){if(null==l)return{};var t={};for(var r in l)if({}.hasOwnProperty.call(l,r)){if(e.indexOf(r)>=0)continue;t[r]=l[r]}return t}(l,e);if(Object.getOwnPropertySymbols){var i=Object.getOwnPropertySymbols(l);for(r=0;r<i.length;r++)t=i[r],e.indexOf(t)>=0||{}.propertyIsEnumerable.call(l,t)&&(n[t]=l[t])}return n}(l,n);return(0,r.Y)("svg",i({xmlns:"http://www.w3.org/2000/svg",width:32,height:32,viewBox:"0 0 32 32","aria-labelledby":t},a),e?(0,r.Y)("title",{id:t},e):null,(0,r.Y)("path",{d:"M2.996 18.636 0 20.434v6.132l6 3.6 5-3 5 3 5-3 5 3 6-3.6v-6.132l-5-3v-5l-5-3v-5l-6-3.6-6 3.6v2.832l2 .033V5.566l4-2.4 4 2.4v3.868l-1.996 1.197 1 1.733L21 11.166l4 2.4v3.868l-4 2.4-2.257-1.354-.971 1.75L20 21.565v3.868l-4 2.4-4-2.4v-1.7l-2-.033v1.733l-4 2.4-4-2.4v-3.868l1.996-1.197-1-1.733ZM22 25.434v-3.868l4-2.4 4 2.4v3.868l-4 2.4-4-2.4Z"}),(0,r.Y)("path",{d:"m11 22.166-6-3.6v-6.132l6-3.6 6 3.6v6.132l-6 3.6Zm4-4.732v-3.868l-4-2.4-4 2.4v3.868l4 2.4 4-2.4Z",className:"euiIcon__fillSecondary"}))}}}]);
//# sourceMappingURL=icon.app_agent.267cd7f1.chunk.js.map